#include "formdesignersession.h"
#include "formmaket.h"
#include "commands.h"
#include "designingarea.h"
#include "widgetmanager.h"
#include "widgetfactory.h"
#include <QFileDialog>
#include <QBuffer>
#include <QDataStream>
#include <QApplication>
#include <QDebug>
FormDesignerSession::FormDesignerSession ( QMdiArea * mdiArea , QObject *parent) :
    QObject (parent)
  , showWidgetName ( true )
  , previewWindow ( 0 )
{
    WM = new WidgetManager ( this ) ;

    connect ( WM , SIGNAL (selectionChanged ()) , this , SIGNAL (selectionChanged ())) ;
    connect ( WM , SIGNAL (structureChanged ()) , this , SIGNAL (structureChanged ())) ;

    undoStack = new QUndoStack ( this ) ;

    maket = new ThreeSectionsMaket ( mdiArea ) ;
    mdiWindow = mdiArea->addSubWindow ( maket ) ;
    maket->init () ;
    maket->setGeometry ( 0 , 0, 640 , 480 ) ;
    maket->show () ;
    connect ( maket , SIGNAL (movingWidgets (QPoint)) , SLOT (moveWidgets (QPoint))) ;
    connect ( maket , SIGNAL (resizeWidgets (QPoint,int)) , SLOT (resizeWidgets (QPoint,int))) ;
    connect ( WM , SIGNAL (resizeWidgets (QPoint,int)) , SLOT (resizeWidgets (QPoint,int))) ;

    connect ( maket , SIGNAL (endMoving ()) , SLOT (endMoving ())) ;
    connect ( maket , SIGNAL (endResizing ()) , SLOT (endResizing ())) ;
    connect ( WM , SIGNAL (endResizing ()) , SLOT (endResizing ())) ;
    connect ( WM , SIGNAL (updateSectionSize (DesigningArea*)) , maket , SLOT (updateSectionSize (DesigningArea*)) ) ;
    connect ( maket , SIGNAL (inFocus (DesigningArea *)) , WM , SLOT (focusChanged (DesigningArea *))) ;
    connect ( maket , SIGNAL (moveWidgetByPaste ( QWidget * , QPoint)) ,
             SLOT (moveWidgetByPaste ( QWidget *, QPoint))) ;

    connect ( maket , SIGNAL (widgetsDragged (QWidget *)) , WM ,
             SLOT (widgetsDragged (QWidget *))) ;

    connect ( maket , SIGNAL (widgetsSelected (QList<QPointer<QWidget> >,bool)) , WM , SLOT (selectWidgets (QList<QPointer<QWidget> >,bool))) ;
    connect ( maket , SIGNAL (clearSelection ()) , WM , SLOT (clearSelection ())) ;

    connect ( maket , SIGNAL (contextMenuSignal (QWidget*)) , this , SIGNAL (contextMenuSignal (QWidget*)) ) ;
    connect ( maket , SIGNAL(widgetTooltip(QWidget*)) , SLOT(widgetToolTipSlot(QWidget*))) ;
    connect ( WM , SIGNAL (propsEdited (QVariantMap)) , SLOT (propsEdited (QVariantMap)) ) ;
    connect ( maket , SIGNAL(leftClicked()) , SIGNAL(leftClicked())) ;
}

QMdiSubWindow *FormDesignerSession::getMdiWindow ()
{
    return mdiWindow ;
}

FormDesignerSession::~FormDesignerSession ()
{

}

void FormDesignerSession::moveWidgets ( QPoint delta )
{
    if ( ! isActive )
        return ;
    undoStack->push ( new MoveCommand ( * WM , delta  ) ) ;
}

void FormDesignerSession::resizeWidgets ( QPoint delta,int i )
{
    if ( ! isActive )
        return ;
    undoStack->push ( new ResizeCommand ( * WM , delta , i) ) ;
}

void FormDesignerSession::endMoving ()
{
    if ( ! isActive )
        return ;
    MoveCommand::endMoving () ;
}

void FormDesignerSession::endResizing ()
{
    if ( ! isActive )
        return ;
    ResizeCommand::endResizing () ;
}

void FormDesignerSession::moveWidgetByPaste ( QWidget * parent , QPoint hotspot )
{
    if ( ! isActive )
        return ;
    undoStack->push ( new MoveAndDelCommand ( * WM , parent, hotspot ) ) ;
}

void FormDesignerSession::deleteItem ( )
{
    if ( ! isActive )
        return ;
    undoStack->push ( new DeleteCommand ( * WM ) ) ;
}

void FormDesignerSession::deleteWidgets ( QList < QPointer <QWidget > > l)
{
    if ( ! isActive )
        return ;
    WM->clearSelection();
    WM->selectWidgets( l , false ) ;
    undoStack->push ( new DeleteCommand ( * WM ) ) ;
}

void FormDesignerSession::propsEdited ( QVariantMap v )
{
    if ( ! isActive )
        return ;
//    qDebug() << v [ "font" ].toString() ;
    undoStack->push ( new EditCommand ( * WM , v ) ) ;
}

void FormDesignerSession::pasteToRoot ()
{
    if ( ! isActive )
        return ;
    QWidget * w = 0 ;
    if ( WM->getCurrentSelList ().isEmpty () )
        w = WM->activeArea () ;
    else
    {
        QWidget * c = WM->getCurrentSelList () [ 0 ] ;
        if ( c && c->acceptDrops () )
            w = c ;
        else
            w = WM->activeArea () ;
    }
    pasteItem ( w , QPoint () ) ;
}

void FormDesignerSession::pasteItem ( QWidget * parent, QPoint p )
{
    if ( ! isActive )
        return ;
    undoStack->push ( new MoveAndDelCommand ( * WM , parent, p ) ) ;
}

void FormDesignerSession::copyItem ()
{
    if ( ! isActive )
        return ;
    WM->copySelected () ;
}

void FormDesignerSession::cutItem ()
{
    if ( ! isActive )
        return ;
    WM->copySelected () ;
    undoStack->push ( new DeleteCommand ( * WM ) ) ;
}

void FormDesignerSession::saveForm (QString fileName)
{
    // copied -> TODO Aleksandr: please add new helper function to avoid copy

    QFile file ( fileName ) ;
    file.open ( QIODevice::WriteOnly ) ;
    QByteArray data = WM->save ( maket ) ;
    QBuffer buf ;
    buf.open ( QIODevice::WriteOnly ) ;
    QDataStream stream ( & buf ) ;
    QVariantMap props ;
    props.insert ( "Header is visible" , maket->isAreaVisible ( "Header" ) ) ;
    props.insert ( "Footer is visible" , maket->isAreaVisible ( "Footer" ) ) ;
    props.insert ( "Grid is enabled" , maket->gridState () ) ;
    props.insert ( "Show cursor pos" , maket->getShowCursorPos() ) ;
    props.insert ( "Show w name" , showWidgetName ) ;
    props.insert ( "Sections is auto resizeble" , maket->isSectionsAutoResizeble () ) ;
    props.insert ( "Maket data" , data ) ;
    props.insert ( "mdiWindow size" , mdiWindow->size () ) ;
    stream << props ;
    file.write ( qCompress( buf.data () ) ) ;
}

void FormDesignerSession::saveForm ()
{
    if ( ! isActive )
        return ;
    QString fileName = QFileDialog::getSaveFileName ( maket, tr ("Save Form") ,
                               "form.maket",
                               tr ("Form designer makets  (*.maket)") ) ;
    QFile file ( fileName ) ;
    file.open ( QIODevice::WriteOnly ) ;
    QByteArray data = WM->save ( maket ) ;
    QBuffer buf ;
    buf.open ( QIODevice::WriteOnly ) ;
    QDataStream stream ( & buf ) ;
    QVariantMap props ;
    props.insert ( "Header is visible" , maket->isAreaVisible ( "Header" ) ) ;
    props.insert ( "Footer is visible" , maket->isAreaVisible ( "Footer" ) ) ;
    props.insert ( "Grid is enabled" , maket->gridState () ) ;
    props.insert ( "Show cursor pos" , maket->getShowCursorPos() ) ;
    props.insert ( "Show w name" , showWidgetName ) ;
    props.insert ( "Sections is auto resizeble" , maket->isSectionsAutoResizeble () ) ;
    props.insert ( "Maket data" , data ) ;
    props.insert ( "mdiWindow size" , mdiWindow->size () ) ;
    stream << props ;
    file.write ( qCompress( buf.data () ) ) ;
}

void FormDesignerSession::loadForm(QString fileName)
{

    // copied -> TODO Aleksandr: please add new helper function to avoid copy
    selectAll () ;
    deleteItem () ;

    QFile file ( fileName ) ;
    file.open ( QIODevice::ReadOnly ) ;
    QByteArray data = qUncompress( file.readAll () ) ;
    QBuffer buf ;
    buf.setData ( data ) ;
    buf.open ( QIODevice::ReadOnly ) ;
    QDataStream stream ( & buf ) ;
    QVariantMap props ;
    stream >> props ;
    maket->setAreaVisible ( "Footer" , props.value ( "Footer is visible" ).toBool () ) ;
    maket->setAreaVisible ( "Header" , props.value ( "Header is visible" ).toBool () ) ;
    maket->setGrid ( props.value ( "Grid is enabled" ).toBool () ) ;
    maket->setSectionsSizePolicy ( props.value ( "Sections is auto resizeble" ).toBool () ? FormMaket::AUTO : FormMaket::FIXED );
    mdiWindow->setGeometry ( QRect ( mdiWindow->geometry ().topLeft () ,
                                   props.value ("mdiWindow size").toSize () ) ) ;
    showWidgetName = props.value ( "Show w name" ).toBool () ;
    maket->setShowCursorPos( props.value ( "Show cursor pos" ).toBool () ) ;
    WM->load ( maket , props.value ( "Maket data" ).toByteArray () ) ;


}

void FormDesignerSession::loadForm ()
{
    if ( ! isActive )
        return ;
    QString fileName = QFileDialog::getOpenFileName (maket, tr ("Open Form"),
                                                     "",
                                                     tr ("Form designer makets  (*.maket)"));
    selectAll () ;
    deleteItem () ;


    QFile file ( fileName ) ;
    file.open ( QIODevice::ReadOnly ) ;
    QByteArray data = qUncompress( file.readAll () ) ;
    QBuffer buf ;
    buf.setData ( data ) ;
    buf.open ( QIODevice::ReadOnly ) ;
    QDataStream stream ( & buf ) ;
    QVariantMap props ;
    stream >> props ;
    maket->setAreaVisible ( "Footer" , props.value ( "Footer is visible" ).toBool () ) ;
    maket->setAreaVisible ( "Header" , props.value ( "Header is visible" ).toBool () ) ;
    maket->setGrid ( props.value ( "Grid is enabled" ).toBool () ) ;
    maket->setSectionsSizePolicy ( props.value ( "Sections is auto resizeble" ).toBool () ? FormMaket::AUTO : FormMaket::FIXED );
    mdiWindow->setGeometry ( QRect ( mdiWindow->geometry ().topLeft () ,
                                   props.value ("mdiWindow size").toSize () ) ) ;
    showWidgetName = props.value ( "Show w name" ).toBool () ;
    maket->setShowCursorPos( props.value ( "Show cursor pos" ).toBool () ) ;
    WM->load ( maket , props.value ( "Maket data" ).toByteArray () ) ;
}

void FormDesignerSession::alignLeft ()
{
    if ( ! isActive )
        return ;
    undoStack->push ( new AlignCommand ( WM , WidgetManager::Left ) ) ;
}

void FormDesignerSession::alignTop ()
{
    if ( ! isActive )
        return ;
    undoStack->push ( new AlignCommand ( WM , WidgetManager::Top ) ) ;
}

void FormDesignerSession::alignRight ()
{
    if ( ! isActive )
        return ;
    undoStack->push ( new AlignCommand ( WM , WidgetManager::Right ) ) ;
}

void FormDesignerSession::alignBottom ()
{
    if ( ! isActive )
        return ;
    undoStack->push ( new AlignCommand ( WM , WidgetManager::Bottom ) ) ;
}

void FormDesignerSession::switchGrid ( bool b )
{
    if ( ! isActive )
        return ;
    maket->setGrid ( b ) ;
    WM->tuneSelectionsGeometry () ;
}

void FormDesignerSession::switchShowCursorPos ( bool b )
{
    if ( ! isActive )
        return ;
    maket->setShowCursorPos ( b ) ;
}

void FormDesignerSession::windowActivated ( QMdiSubWindow * w )
{
    if ( w && w->widget () == maket )
    {
        //if( isActive == false )
        //{
            emit wmChanged ( WM )  ;
            emit maketChanged ( maket ) ;
            emit activated ( this ) ;
            emit structureChanged() ;
        //}
        isActive = true ;
        connect (WidgetFactory::instance() , SIGNAL(pasteNeeded()) , SLOT(pasteToRoot()) , Qt::UniqueConnection) ;
        connect (WidgetFactory::instance() , SIGNAL(deleteNeeded(QList < QPointer <QWidget > >)) ,
                 SLOT(deleteWidgets(QList<QPointer<QWidget> >)) , Qt::UniqueConnection) ;

    }
    else if ( w == 0 )
    {

        emit activated ( 0 ) ;
        emit wmChanged ( 0 )  ;
        emit maketChanged ( 0 ) ;
    }
    else
        isActive = false ;
}

void FormDesignerSession::selectAll ()
{
    if ( ! isActive )
        return ;
    WM->selectAll () ;
}

void FormDesignerSession::undo ()
{
    if ( ! isActive )
        return ;
    undoStack->undo () ;
}

void FormDesignerSession::redo ()
{
    if ( ! isActive )
        return ;
    undoStack->redo () ;
}

WidgetManager * FormDesignerSession::getWM ()
{
    return WM ;
}

void FormDesignerSession::selectItems ( QList<QPointer<QWidget> > l ,bool b )
{
    if ( ! isActive )
        return ;
    WM->selectWidgets ( l , b ) ;
}

bool FormDesignerSession::gridState ()
{
    return maket->gridState () ;
}

void FormDesignerSession::switchHeader ( bool b )
{
    if ( ! isActive )
        return ;
    maket->setAreaVisible ( "Header" , b ) ;
    emit structureChanged () ;
    QApplication::processEvents () ;
    maket->updateSectionSize ( 0 ) ;
}

void FormDesignerSession::switchFooter ( bool b )
{
    if ( ! isActive )
        return ;
    maket->setAreaVisible ( "Footer" , b ) ;
    emit structureChanged () ;
    QApplication::processEvents () ;

    maket->updateSectionSize ( 0 ) ;
}

QSize FormDesignerSession::getFormSize ()
{
    return maket->size () ;
}

void FormDesignerSession::setMaketSectionsAutoResize ( bool b )
{
    if ( ! isActive )
        return ;
    maket->setSectionsSizePolicy ( b ? FormMaket::AUTO : FormMaket::FIXED ) ;
}

bool FormDesignerSession::isMaketSectionsAutoresize ()
{
    return maket->isSectionsAutoResizeble () ;
}

void FormDesignerSession::copyProperties ()
{
    if ( ! isActive )
        return ;
    WM->copyProperties() ;
}

void FormDesignerSession::assignProperties ()
{
    if ( ! isActive )
        return ;
    undoStack->push ( new EditCommand ( * WM , WM->getPropBuffer() ) ) ;
}

void FormDesignerSession::morphWidgets( QString n )
{
    if ( ! isActive )
        return ;
    undoStack->push ( new MorphCommand ( * WM , n ) ) ;
}

void FormDesignerSession::startTabOrderEditing ()
{
    if ( ! isActive )
        return ;
    WM->startTabOrderEditing () ;
}

void FormDesignerSession::makeGeometryRelated ()
{
    if ( ! isActive )
        return ;
    WM->makeGeometryRelated () ;
}

void FormDesignerSession::makeGeometryUnRelated ()
{
    if ( ! isActive )
        return ;
    WM->makeGeometryUnRelated () ;
}

QList < QPointer < QWidget > > FormDesignerSession::getSelectedWidgets ()
{
    return WM->getCurrentSelList() ;
}

void FormDesignerSession::moveLeft ()
{
    moveWidgets ( QPoint ( - maket->getGridStep() , 0  ) ) ;
    endMoving () ;
}
void FormDesignerSession::moveTop ()
{
    moveWidgets ( QPoint ( 0 , - maket->getGridStep() ) ) ;
    endMoving () ;
}
void FormDesignerSession::moveRight ()
{
    moveWidgets ( QPoint ( maket->getGridStep() , 0 ) ) ;
    endMoving () ;
}
void FormDesignerSession::moveBottom ()
{
    moveWidgets ( QPoint ( 0 , maket->getGridStep() ) ) ;
    endMoving () ;
}

void FormDesignerSession::widgetToolTipSlot( QWidget * w )
{
    if ( showWidgetName )
        WM->showTooltipForWidget ( w ) ;
}

void FormDesignerSession::switchShowWidgetName( bool b )
{
    showWidgetName = b ;
}

bool FormDesignerSession::isShowWidgetName ()
{
    return showWidgetName ;
}

bool FormDesignerSession::getShowCursorPos ( )
{
    return maket->getShowCursorPos () ;
}

void FormDesignerSession::moveToBack ()
{
    if ( ! isActive )
        return ;
    WM->moveToBack () ;
}

void FormDesignerSession::moveToFront ()
{
    if ( ! isActive )
        return ;
    WM->moveToFront () ;
}

void FormDesignerSession::setSameHeight()
{
    if ( ! isActive )
        return ;
    WM->setSameHeight () ;
}

void FormDesignerSession::setSameWidth()
{
    if ( ! isActive )
        return ;
    WM->setSameWidth () ;
}

void FormDesignerSession::setSameSize()
{
    if ( ! isActive )
        return ;
    WM->setSameSize () ;
}

void FormDesignerSession::showPreview ()
{
    if ( ! isActive )
        return ;
    if ( previewWindow )
        previewWindow->deleteLater () ;
    ThreeSectionsMaket * p = new ThreeSectionsMaket ( 0 ) ;
    connect ( p , SIGNAL(closing()) , p , SLOT(deleteLater()) ) ;
    p->init () ;

    p->hide() ;
    p->setAreaVisible( "Header" , maket->isAreaVisible( "Header" ) ) ;
    p->setAreaVisible( "Footer" , maket->isAreaVisible( "Footer" ) ) ;
    WM->load ( p , WM->save( maket ) ) ;

    p->setGeometry( maket->geometry() ) ;
    p->setMode ( 1 ) ;
    WM->applyPropertiesReplacements( p ) ;
    p->show () ;
    p->updateSectionSize ( 0 ) ;
    p->setSplitterPositions ( maket->splitterPositions() ) ;
    previewWindow = p ;
}

void FormDesignerSession::setNoLayout()
{
    if ( ! isActive )
        return ;
    WM->setNoLayout() ;
}

void FormDesignerSession::setHorLayout()
{
    if ( ! isActive )
        return ;
    WM->setHorLayout() ;
}

void FormDesignerSession::setVertLayout()
{
    if ( ! isActive )
        return ;
    WM->setVertLayout() ;
}
