/*
    This file containes declaration of  FormDesignerSession class.
    FormDesignerSession provides "one document" entity.
    It contains FormMaket, WidgetManager, UndoStack and MDI subwindow
*/

#ifndef FORMDESIGNERSESSION_H
#define FORMDESIGNERSESSION_H

#include <QObject>
#include <QUndoStack>
#include <QMdiSubWindow>
#include <QVariantMap>
#include <QPointer>
#include <QMdiArea>
class WidgetManager ;
class FormMaket ;

class FormDesignerSession : public QObject
{
    Q_OBJECT
    QUndoStack * undoStack;
    FormMaket * maket ;
    WidgetManager * WM ;
    bool isActive ;
    QMdiSubWindow * mdiWindow ;
    bool showWidgetName ;
    QPointer < FormMaket > previewWindow ;
public:
    explicit FormDesignerSession (QMdiArea *mdiArea , QObject *parent = 0) ;
    //returns pointe to mdi-subwindow
    QMdiSubWindow *getMdiWindow () ;
    ~FormDesignerSession () ;
    WidgetManager * getWM () ;
    // Returns true if grid on FormMaket sections is on, otherwise returns false
    bool gridState () ;
    // Returns size of FormMaket
    QSize getFormSize () ;
    // Returns true if sections of FormMaket automatically change size according to Form size.
    // Otherwise returns false;
    bool isMaketSectionsAutoresize () ;
    //returns selected widgets
    QList < QPointer < QWidget > > getSelectedWidgets () ;
    //returns true if tooltip with widget name is shown
    bool isShowWidgetName () ;
    // returns true if cursor showing is enabled for maket
    bool getShowCursorPos ( ) ;

    void loadForm (QString fileName);
    void saveForm (QString fileName);

signals:
    // Emitted when Sessions becomes active and FormDesigner current WidgetManager changes
    void wmChanged ( WidgetManager *  ) ;
    // Emitted when Sessions becomes active and FormDesigner current FormMaket changes
    void maketChanged ( FormMaket * maket ) ;
    // Emitted when widgets selection changes
    void selectionChanged () ;
    // Emitted when widgets hierarhy changes
    void structureChanged () ;
    // Emitted when user executes context menu for FormMaket of this Session
    void contextMenuSignal ( QWidget * ) ;
    // Emitted when session becomes active
    void activated ( FormDesignerSession *  ) ;

    void leftClicked () ;

private slots:
    // Called when maket reports about widgets moving
    void moveWidgets ( QPoint delta ) ;
    // Called when maket reports about widgets resizing
    void resizeWidgets ( QPoint delta,int ) ;
    void endMoving () ;
    void endResizing () ;
    // Called when widgets should be pasted into section
    void pasteItem ( QWidget * , QPoint ) ;

    // Called when maket reports about widgets dropping
    void moveWidgetByPaste ( QWidget * parent , QPoint hotspot ) ;
    // Called when it is time to show tooltip for widget
    void widgetToolTipSlot( QWidget * w ) ;

    void deleteWidgets ( QList < QPointer <QWidget > > ) ;
public slots :
    // called when session should become active
    void windowActivated ( QMdiSubWindow * w ) ;
    // called when "delete" action must be triggered
    void deleteItem ( ) ;
    // called when "select all" action must be triggered
    void selectAll () ;
    // called when "copy" action must be triggered
    void copyItem () ;
    // called when "cut" action must be triggered
    void cutItem () ;
    // called when "switch grid" action must be triggered
    void switchGrid ( bool ) ;
    // called when "switch cursor pos showing" action must be triggered
    void switchShowCursorPos ( bool b ) ;
    // called when "paste" action must be triggered
    void pasteToRoot () ;
    // called when properties of selected widgets must be changed
    void propsEdited ( QVariantMap v ) ;
    // called when "save" action must be triggered
    void saveForm () ;
    // called when "load" action must be triggered
    void loadForm () ;
    // called when align actions must be triggered
    void alignLeft () ;
    void alignTop () ;
    void alignRight () ;
    void alignBottom () ;
    // called when move actions must be triggered
    void moveLeft () ;
    void moveTop () ;
    void moveRight () ;
    void moveBottom () ;
    // called when "undo" action must be triggered
    void undo () ;
    // called when "redo" action must be triggered
    void redo () ;
    // called when widgets of active session should become selected
    void selectItems ( QList<QPointer<QWidget> >,bool ) ;
    // called when visibility of FormMaket header should be changed
    void switchHeader ( bool b ) ;
    // called when visibility of FormMaket footer should be changed
    void switchFooter ( bool b ) ;
    // called when resizing mode of Maket sections should be changed
    void setMaketSectionsAutoResize ( bool b ) ;
    // called when properties of selected widget should be remembered
    void copyProperties () ;
    // called when remembered properties should be assigned to selected widgets
    void assignProperties () ;
    // called when widgets should morph to desired class
    void morphWidgets( QString n ) ;
    // called when tab order editing should start
    void startTabOrderEditing () ;
    // converts current coordinates of selected widgets to relative coordinates
    void makeGeometryRelated () ;

    void makeGeometryUnRelated () ;
    // sets tooltip showing mode
    void switchShowWidgetName( bool b ) ;
    // makes first selected widget first in z-order
    void moveToBack () ;
    // makes first selected widget last in z-order

    void moveToFront () ;

    void setSameHeight() ;
    void setSameWidth() ;
    void setSameSize() ;
    void showPreview () ;

    void setNoLayout() ;
    void setHorLayout() ;
    void setVertLayout() ;
} ;

#endif // FORMDESIGNERSESSION_H
